//
//  NSUserDefaults+AXAdd.m
//  AXKit
//
//  Created by xaoxuu on 17/03/2017.
//  Copyright © 2017 Titan Studio. All rights reserved.
//

#import "NSUserDefaults+AXAdd.h"
#import "NSError+AXAdd.h"
#import "NSLog+AXAdd.h"
#import "_AXKitError.h"
#import "NSObject+AXAdd.h"
#import "NSObject+AXJsonAdd.h"

@implementation NSError (AXUserDefaults)

+ (instancetype)axkit_errorWithReason:(NSString *(^)(void))reason{
    return [self axkit_errorWithCode:AXKitErrorCodeObjectForKeyNotFound reason:^NSString * _Nonnull{
        return reason?reason():nil;
    }];
}

@end

@implementation NSUserDefaults (AXAdd)

+ (NSUserDefaults *)standard{
    return NSUserDefaults.standardUserDefaults;
}

#pragma mark - read

+ (nullable AXResult *(^)(NSString *key))objectResult{
    return ^id (NSString *key){
        return [AXResult resultWithIdResult:^id _Nonnull(NSError * _Nullable __autoreleasing * _Nullable error) {
            return [self.standard objectForKey:key];
        }];
    };
}
- (nullable AXResult *(^)(NSString *key))objectResult{
    return ^id (NSString *key){
        return [AXResult resultWithIdResult:^id _Nonnull(NSError * _Nullable __autoreleasing * _Nullable error) {
            return [self objectForKey:key];
        }];
    };
}

+ (nullable id (^)(NSString *key))objectForKey{
    return ^id (NSString *key){
        return [self.standard objectForKey:key];
    };
}
- (nullable id (^)(NSString *key))objectForKey{
    return ^id (NSString *key){
        return [self objectForKey:key];
    };
}

+ (nullable NSString *(^)(NSString *key))stringForKey{
    return ^id (NSString *key){
        return NSString.parse([self.standard objectForKey:key]);
    };
}
- (nullable NSString *(^)(NSString *key))stringForKey{
    return ^id (NSString *key){
        return NSString.parse([self objectForKey:key]);
    };
}

+ (nullable NSNumber *(^)(NSString *key))numberForKey{
    return ^id (NSString *key){
        return NSNumber.safe([self.standard objectForKey:key]);
    };
}
- (nullable NSNumber *(^)(NSString *key))numberForKey{
    return ^id (NSString *key){
        return NSNumber.safe([self objectForKey:key]);
    };
}

+ (nullable NSArray *(^)(NSString *key))arrayForKey{
    return ^id (NSString *key){
        return NSArray.parse([self.standard objectForKey:key]);
    };
}
- (nullable NSArray *(^)(NSString *key))arrayForKey{
    return ^id (NSString *key){
        return NSArray.parse([self objectForKey:key]);
    };
}

+ (nullable NSDictionary *(^)(NSString *key))dictionaryForKey{
    return ^id (NSString *key){
        return NSDictionary.parse([self.standard objectForKey:key]);
    };
}
- (nullable NSDictionary *(^)(NSString *key))dictionaryForKey{
    return ^id (NSString *key){
        return NSDictionary.parse([self objectForKey:key]);
    };
}

+ (nullable NSData *(^)(NSString *key))dataForKey{
    return ^id (NSString *key){
        return NSData.parse([self.standard objectForKey:key]);
    };
}
- (nullable NSData *(^)(NSString *key))dataForKey{
    return ^id (NSString *key){
        return NSData.parse([self objectForKey:key]);
    };
}

+ (nullable UIImage *(^)(NSString *key))imageForKey{
    return ^id (NSString *key){
        return self.standard.imageForKey(key);
    };
}
- (nullable UIImage *(^)(NSString *key))imageForKey{
    return ^id (NSString *key){
        NSData *data = NSData.safe([self objectForKey:key]);
        if (data) {
            return [UIImage imageWithData:data];
        } else {
            return nil;
        }
    };
}

+ (nullable NSURL *(^)(NSString *key))URLForKey{
    return ^NSURL *(NSString *key){
        return [self.standard URLForKey:key];
    };
}
- (nullable NSURL *(^)(NSString *key))URLForKey{
    return ^NSURL *(NSString *key){
        return [self URLForKey:key];
    };
}

#pragma mark - write

+ (void (^)(id , NSString * _Nonnull))setObjectForKey{
    return ^(id obj, NSString *key){
        [self.standard setObject:obj forKey:key];
        [self.standard synchronize];
    };
}
- (void (^)(id , NSString * _Nonnull))setObjectForKey{
    return ^(id obj, NSString *key){
        [self setObject:obj forKey:key];
        [self synchronize];
    };
}

+ (void)ax_caches:(void (^)(NSUserDefaults *defaultUser))action{
    action(self.standard);
    [self.standard synchronize];
}

- (void)ax_caches:(void (^)(NSUserDefaults *user))action{
    action(self);
    [self synchronize];
}

#pragma mark - remove

+ (void (^)(NSString *key))removeObjectForKey{
    return ^(NSString *key){
        self.standard.removeObjectForKey(key);
    };
}
- (void (^)(NSString *key))removeObjectForKey{
    return ^(NSString *key){
        [self removeObjectForKey:key];
        [self synchronize];
    };
}


+ (void (^)(NSString *name))removePersistentDomainForName{
    return ^(NSString *name){
        [self.standard removePersistentDomainForName:name];
    };
}

- (void (^)(NSString *name))removePersistentDomainForName{
    return ^(NSString *name){
        [self removePersistentDomainForName:name];
    };
}

+ (void (^)(void))removeDefaultPersistentDomain{
    return ^{
        [self.standard removePersistentDomainForName:[[NSBundle mainBundle] bundleIdentifier]];
    };
}
- (void (^)(void))removeDefaultPersistentDomain{
    return ^{
        [self removePersistentDomainForName:[[NSBundle mainBundle] bundleIdentifier]];
    };
}


@end


